;;;
;;;  Macro library for widget
;;;
;;; @(#)widget.scm 1.0.0 94/02/15 (Shiro Kawai, kawai@sail.t.u-tokyo.ac.jp)
;;; Copyright (C) 1993 by Shiro Kawai

;; WIDGET WIDGETNAME CLASS OPTIONS...
;;   Returns a procedure, which takes one argument (parent widget),
;;   creates a widget specified by OPTIONS and return created widget.
;;
;;   OPTIONS is a list, one of followings;
;;
;;     (:resources NAME VAL ...)
;;		Specify resources.
;;
;;     (:instantiation PROC)
;;		Use PROC for instantiation procedure.  By default,
;;		widget are instantiated by xt:create-managed-widget.
;;		To change that, use this option.   PROC must be a
;;		procedure which takes three or more arguments, as
;;		xt:create-managed-widget.  The most popular case of
;;		using this option is to create shell widget: 
;;		 (:instantiation xt:create-popup-shell)
;;
;;     (:children CHILD ...)
;;		Specify children of this widget.  Each CHILD must
;;		be a procedure which takes one argument, parent widget,
;;		and returns created child widget.  You can write
;;		widget macro form as CHILD.
;;			
;;     (:callbacks CALLBACKNAME CALLBACKPROC ...)
;;		Specify callbacks for this widget.  CALLBACKNAME specifies
;;		type of callback, and CALLBACKPROC is procedure which will
;;		be invoked with one argument, widget, when callback is
;;		invoked.
;;		You cannot get other information except widget in which
;;		the callback procedure invoked.  To obtain other information,
;;		such as a selected item of list widget, use xaw:add-callback
;;		function in :init option.
;;
;;     (:init PROC ...)
;;		Each PROC are called after target widget and children are
;;		created.  You can write miscellaneous stuff here.  Two
;;		arguments are passed to PROC; one is widget object just
;;		created, and the other is a list of child widgets.
;;
;;
;;   Note that widget instantiation is done only once.  Applying widget
;;   form more than once, you get the same widget created first.
;;
;;   Since widget macro uses nested syntax-rules, macro expansion will be
;;   too slow to use interactively.  widgetex.scm in this directory generates
;;   forms where widget macros are expanded.
;;;(require 'macros-that-work)
;;(require 'macro)
(if (not (memq 'macro *features*))
    (require 'macro-by-example))

(define-syntax widget
  (syntax-rules
   ()
   ((widget wname class options ...)
    (widget:sub wname class
      wname xt:create-managed-widget () () () ()
      options ...))))

(define-syntax widget:sub
  (syntax-rules
   (:resources :children :callbacks :init :instantiation)
   ;; Analyze options
   ((widget:sub pname class
      wn ins res chi cal ini (:resources args ...) options ...)
    (widget:sub pname class
      wn ins (args ...) chi cal ini options ...))
   ((widget:sub pname class
      wn ins res chi cal ini (:children args ...) options ...)
    (widget:sub pname class
      wn ins res (args ...) cal ini options ...))
   ((widget:sub pname class
      wn ins res chi cal ini (:callbacks args ...) options ...)
    (widget:sub pname class
      wn ins res chi (args ...) ini options ...))
   ((widget:sub pname class
      wn ins res chi cal ini (:init forms ...) options ...)
    (widget:sub pname class
      wn ins res chi cal (forms ...) options ...))
   ((widget:sub pname class
      wn ins res chi cal ini (:instantiation proc) options ...)
    (widget:sub pname class
      wn proc res chi cal ini options ...))
   ;; Generate code
   ((widget:sub pname class wn ins res chi cal ini)
    (let ((self #f))
      (lambda (parent)
	(if self self
	    (begin
	      (set! self (widget:create wn class parent ins res))
	      (let* ((child (widget:map-child self chi)))
		(widget:map-cb self cal)
		(widget:map-init self child ini)
		self))))))
   ))

(define-syntax widget:create
  (syntax-rules
   ()
   ((widget:create name class parent ins () resources ...)
    (ins name class parent resources ...))
   ((widget:create name class parent ins (resourcename resourceval rest ...)
		   resources ...)
    (widget:create name class parent ins (rest ...)
		   resourcename resourceval resources ...))))

(define-syntax widget:map-child
  (syntax-rules ()
		((widget:map-child s ()) '())
		((widget:map-child s ((child proc) rest ...))
		 (cons (let ((c (child s)))
			 (proc s c)
			 c)
		       (widget:map-child s (rest ...))))
		((widget:map-child s (child rest ...))
		 (cons (child s)
		       (widget:map-child s (rest ...))))))

(define-syntax widget:map-cb
  (syntax-rules ()
		((widget:map-cb s ()) '())
		((widget:map-cb s (name proc rest ...))
		 (begin (xt:add-callback s name proc)
			(widget:map-cb s (rest ...))))))

(define-syntax widget:map-init
  (syntax-rules ()
		((widget:map-init s c ()) '())
		((widget:map-init s c (form forms ...))
		 (begin (form s c)
			(widget:map-init s c (forms ...))))))

